@@ -1,3 +1,4 @@
+// -*- C++ -*-
 //
 // Class: DVIWidget
 //
@@ -9,25 +10,25 @@
 #ifndef _DVIWIDGET_H_
 #define _DVIWIDGET_H_
 
-#include <qwidget.h> 
-
 #include "documentWidget.h"
 
-class CenteringScrollview;
-class documentPageCache;
+class PageView;
+class DocumentPageCache;
 class QPaintEvent;
 class QMouseEvent;
+class QWidget;
 class textSelection;
 
-class DVIWidget : public documentWidget
+
+class DVIWidget : public DocumentWidget
 {
   Q_OBJECT
 
 public: 
-  DVIWidget(QWidget* parent, CenteringScrollview* sv, QSize size,
-            documentPageCache* cache, textSelection* documentSelection, const char* name);
+  DVIWidget(QWidget* parent, PageView* sv, DocumentPageCache* cache, const char* name);
+
 signals:
-  void SRCLink(const QString&, QMouseEvent* e, documentWidget*);
+  void SRCLink(const QString&, QMouseEvent* e, DocumentWidget*);
 
 private:
   virtual void mousePressEvent(QMouseEvent* e);
