@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <qdom.h>
@@ -297,7 +297,7 @@ MrmlViewItem::MrmlViewItem( const KURL& url, const KURL& thumbURL,
     setMouseTracking( true );
 
     m_combo = new KComboBox( this );
-    QToolTip::add( m_combo, i18n("You can refine queries by giving feedback about the current result and pressing the Search-button again.") );
+    QToolTip::add( m_combo, i18n("You can refine queries by giving feedback about the current result and pressing the Search button again.") );
     m_combo->insertItem( i18n("Relevant"),   Relevant );
     m_combo->insertItem( i18n("Neutral"),    Neutral );
     m_combo->insertItem( i18n("Irrelevant"), Irrelevant );
@@ -399,7 +399,7 @@ void MrmlViewItem::mouseMoveEvent( QMouseEvent *e )
     else {
         if ( ownCursor() ) {
             unsetCursor();
-            emit view()->onItem( QString::null );
+            emit view()->onItem( KURL() );
         }
     }
 
