@@ -1105,11 +1105,19 @@ void kpDocument::setSelection (const kpSelection &selection)
     const bool isTextChanged = (m_mainWindow->toolIsTextTool () !=
                                 (selection.type () == kpSelection::Text));
 
-    // (we don't change the Selection Tool if the new selection's
-    //  shape is different to the tool's because all the Selection
-    //  Tools act the same, except for what would be really irritating
-    //  if it kept changing whenever you paste an image - drawing the
-    //  selection region)
+    // We don't change the Selection Tool if the new selection's
+    // shape is merely different to the current tool's (e.g. rectangular
+    // vs elliptical) because:
+    //
+    // 1. All image selection tools support editing selections of all the
+    //    different shapes anyway.
+    // 2. Suppose the user is trying out different drags of selection borders
+    //    and then decides to paste a differently shaped selection before continuing
+    //    to try out different borders.  If the pasting were to switch to
+    //    a differently shaped tool, the borders drawn after the paste would
+    //    be using a new shape rather than the shape before the paste.  This
+    //    could get irritating so we don't do the switch.
+    //
     if (m_mainWindow &&
         (!m_mainWindow->toolIsASelectionTool () || isTextChanged))
     {
@@ -1138,10 +1146,20 @@ void kpDocument::setSelection (const kpSelection &selection)
 
     if (m_selection)
     {
+        // TODO: Emitting this, before setting the new selection, is bogus
+        //       since it would redraw the old selection.
+        //
+        //       Luckily, this doesn't matter thanks to the
+        //       kpViewManager::setQueueUpdates() call above.
         if (m_selection->pixmap ())
             slotContentsChanged (m_selection->boundingRect ());
         else
+            // TODO: Should emit contentsChanged() instead?
+            //       I don't think it matters since contentsChanged() is
+            //       connected to updateViews() anyway (see
+            //       kpMainWindow::setDocument ()).
             vm->updateViews (m_selection->boundingRect ());
+
         delete m_selection;
     }
 
@@ -1187,8 +1205,18 @@ void kpDocument::setSelection (const kpSelection &selection)
     if (m_selection->pixmap ())
         slotContentsChanged (m_selection->boundingRect ());
     else
+        // TODO: Should emit contentsChanged() instead?
+        //       I don't think it matters since contentsChanged() is
+        //       connected to updateViews() anyway (see
+        //       kpMainWindow::setDocument ()).
         vm->updateViews (m_selection->boundingRect ());
 
+    // There's no need to disconnect() the old selection since we:
+    //
+    // 1. Connect our _copy_ of the given selection.
+    // 2. We delete our copy when setSelection() is called again.
+    //
+    // See code above for both.
     connect (m_selection, SIGNAL (changed (const QRect &)),
              this, SLOT (slotContentsChanged (const QRect &)));
 
@@ -1395,6 +1423,8 @@ bool kpDocument::selectionCopyOntoDocument (bool useTransparentPixmap)
 
     if (!sel->isText ())
     {
+        // We can't use kpSelection::paint() since that always uses the
+        // transparent pixmap.
         paintPixmapAt (useTransparentPixmap ? sel->transparentPixmap () : sel->opaquePixmap (),
                        boundingRect.topLeft ());
     }
@@ -1404,6 +1434,8 @@ bool kpDocument::selectionCopyOntoDocument (bool useTransparentPixmap)
         sel->paint (m_pixmap, rect ());
     }
 
+    slotContentsChanged (boundingRect);
+
     return true;
 }
 
@@ -1428,9 +1460,8 @@ QPixmap kpDocument::pixmapWithSelection () const
     #endif
         QPixmap output = *m_pixmap;
 
-        kpPixmapFX::paintPixmapAt (&output,
-                                   m_selection->topLeft (),
-                                   m_selection->transparentPixmap ());
+        m_selection->paint (&output, rect ());
+
         return output;
     }
     else
