@@ -360,6 +360,11 @@ public:
     */
    virtual void readAttributes( const PMXMLHelper& h );
 
+   /**
+    * Returns the list of known properties
+    */
+   QStringList properties( ) const;
+
    /**
     * Sets a property and returns true if successful
     */
