@@ -14,8 +14,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #ifndef SVGElementImpl_H
@@ -28,7 +28,6 @@
 #include <qptrlist.h>
 
 #include <dom/dom_string.h>
-#include <kdemacros.h>
 
 #include "SVGEcma.h"
 
@@ -42,7 +41,7 @@ namespace KJS
 	class Value;
 	class UString;
 	class ExecState;
-};
+}
 
 namespace KSVG
 {
@@ -54,9 +53,9 @@ class SVGEventListener;
 class SVGSVGElementImpl;
 class SVGMouseEventImpl;
 class SVGRegisteredEventListener;
-class KDE_EXPORT SVGElementImpl : virtual public DOM::DomShared,
-								  public DOM::Element,
-								  public SVGDOMElementBridge
+class SVGElementImpl : virtual public DOM::DomShared,
+							   public DOM::Element,
+							   public SVGDOMElementBridge
 {
 public:
 	SVGElementImpl(DOM::ElementImpl *impl);
@@ -129,7 +128,7 @@ public:
 
 	// Element creation
 	typedef SVGElementImpl *(*FactoryFn)(DOM::ElementImpl *impl);
-	class KDE_EXPORT Factory
+	class Factory
 	{
 	public:
 		static Factory *self()
@@ -155,9 +154,9 @@ public:
 		}
 
 	private:
-		KDE_NO_EXPORT Factory() { }
-		KDE_NO_EXPORT Factory(const Factory &rhs);
-		KDE_NO_EXPORT Factory &operator=(const Factory &rhs);
+		Factory() { }
+		Factory(const Factory &rhs);
+		Factory &operator=(const Factory &rhs);
 
 		static Factory *m_instance;
 
@@ -220,7 +219,7 @@ public:
 	void putValueProperty(KJS::ExecState *exec, int token, const KJS::Value &value, int attr);
 };
 
-};
+}
 
 KSVG_DEFINE_PROTOTYPE(SVGElementImplProto)
 KSVG_IMPLEMENT_PROTOFUNC(SVGElementImplProtoFunc, SVGElementImpl)
