@@ -1,3 +1,4 @@
+// -*- C++ -*-
 #ifndef kmultipageINTERFACE_H
 #define kmultipageINTERFACE_H
  
@@ -8,9 +9,9 @@ class kmultipageInterface : virtual public DCOPObject
   K_DCOP
        
   k_dcop:
-   virtual ASYNC   jumpToReference(QString reference) = 0;
+   virtual ASYNC   jumpToReference(const QString& reference) = 0;
    virtual QString name_of_current_file() = 0;
-   virtual bool    is_file_loaded(QString filename) = 0;
+   virtual bool    is_file_loaded(const QString& filename) = 0;
 };
  
 #endif
